@@ -1,37 +1,8 @@
-=head1 NAME
-
-CPAN::Version - utility functions to compare CPAN versions
-
-=head1 SYNOPSIS
-
-  use CPAN::Version;
-
-  CPAN::Version->vgt("1.1","1.1.1");    # 1
-
-  CPAN::Version->vcmp("1.1","1.1.1");   # 1
-
-  CPAN::Version->readable(v1.2.3);      # "v1.2.3"
-
-  CPAN::Version->vstring("v1.2.3");     # v1.2.3
-
-  CPAN::Version->float2vv(1.002003);    # "v1.2.3"
-
-=head1 DESCRIPTION
-
-This module mediates between some version that perl sees in a package
-and the version that is published by the CPAN indexer.
-
-It's only written as a helper module for both CPAN.pm and CPANPLUS.pm.
-
-As it stands it predates version.pm but has the same goal: make
-version strings visible and comparable.
-
-=cut
-
 package CPAN::Version;
 
+use strict;
 use vars qw($VERSION);
-$VERSION = sprintf "%.2f", substr(q$Rev: 231 $,4)/100;
+$VERSION = sprintf "%.6f", substr(q$Rev: 950 $,4)/1000000 + 5.4;
 
 # CPAN::Version::vcmp courtesy Jost Krieger
 sub vcmp {
@@ -41,17 +12,23 @@ sub vcmp {
 
   return 0 if $l eq $r; # short circuit for quicker success
 
+  for ($l,$r) {
+      s/_//g;
+  }
+  CPAN->debug("l[$l] r[$r]") if $CPAN::DEBUG;
   for ($l,$r) {
       next unless tr/.// > 1;
       s/^v?/v/;
       1 while s/\.0+(\d)/.$1/;
   }
+  CPAN->debug("l[$l] r[$r]") if $CPAN::DEBUG;
   if ($l=~/^v/ <=> $r=~/^v/) {
       for ($l,$r) {
           next if /^v/;
           $_ = $self->float2vv($_);
       }
   }
+  CPAN->debug("l[$l] r[$r]") if $CPAN::DEBUG;
 
   return (
           ($l ne "undef") <=> ($r ne "undef") ||
@@ -71,6 +48,11 @@ sub vgt {
   $self->vcmp($l,$r) > 0;
 }
 
+sub vlt {
+  my($self,$l,$r) = @_;
+  0 + ($self->vcmp($l,$r) < 0);
+}
+
 sub vstring {
   my($self,$n) = @_;
   $n =~ s/^v// or die "CPAN::Version::vstring() called with invalid arg [$n]";
@@ -128,6 +110,45 @@ sub readable {
 
 __END__
 
+=head1 NAME
+
+CPAN::Version - utility functions to compare CPAN versions
+
+=head1 SYNOPSIS
+
+  use CPAN::Version;
+
+  CPAN::Version->vgt("1.1","1.1.1");    # 1 bc. 1.1 > 1.001001
+
+  CPAN::Version->vlt("1.1","1.1");      # 0 bc. 1.1 not < 1.1
+
+  CPAN::Version->vcmp("1.1","1.1.1");   # 1 bc. first is larger
+
+  CPAN::Version->vcmp("1.1.1","1.1");   # -1 bc. first is smaller
+
+  CPAN::Version->readable(v1.2.3);      # "v1.2.3"
+
+  CPAN::Version->vstring("v1.2.3");     # v1.2.3
+
+  CPAN::Version->float2vv(1.002003);    # "v1.2.3"
+
+=head1 DESCRIPTION
+
+This module mediates between some version that perl sees in a package
+and the version that is published by the CPAN indexer.
+
+It's only written as a helper module for both CPAN.pm and CPANPLUS.pm.
+
+As it stands it predates version.pm but has the same goal: make
+version strings visible and comparable.
+
+=head1 LICENSE
+
+This program is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
+=cut
+
 # Local Variables:
 # mode: cperl
 # cperl-indent-level: 2
